﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using Newtonsoft.Json.Linq;

namespace Tetris.Models; 

/* It parses the JSON file and returns the shape and color of a tetrominoe */
public class TetrominoeParser {
    private static JObject _content;
    
    private static JObject content {
        get {
            if (_content == null)
                _content = GetContent();

            return _content;
        }
    }

    /// <summary>
    /// It takes a name, and returns a tetrominoe shape
    /// </summary>
    /// <param name="String">name - The name of the shape to get.</param>
    /// <returns>
    /// The shape of the tetrominoe as an 2D array of booleans.
    /// </returns>
    public static bool[,] GetShape(String name) {
        if (!content.ContainsKey(name))
            throw new Exception("Invalid name");

        String s = content[name]["shape"].ToString();

        int width = s.IndexOf('\n');
        if (width < 0)
            width = s.Length;
        int height = s.Count(c => c == '\n')+1;

        bool[,] g = new bool[width, height];

        int x = 0;
        int y = 0;
        for (int i = 0; i < s.Length; i++, x++) {
            char c = Char.ToLower(s[i]);
            if (c == 'x')
                g[x, y] = true;
            else if (c == '-')
                g[x, y] = false;
            else if (c == '\n') {
                x = -1;
                y++;
            }
        }

        return g;
    }

    /// <summary>
    /// It takes a name, checks if it's in the dictionary, and if it is, it returns the color
    /// </summary>
    /// <param name="String">The name of the color you want to get.</param>
    /// <returns>
    /// A color object
    /// </returns>
    public static Color GetColor(String name) {
        if (!content.ContainsKey(name))
            throw new Exception("Invalid name");
        
        String s = content[name]["color"].ToString();

        return ColorTranslator.FromHtml(s);
    }

    /// <summary>
    /// It returns a list of all the properties in the content object
    /// </summary>
    /// <returns>
    /// A list of all the property names in the content object.
    /// </returns>
    public static List<String> List() {
        return content.Properties().Select(p => p.Name).ToList();
    }


    /// <summary>
    /// It reads the contents of the tetrominoes.json file and returns it as a JObject
    /// </summary>
    /// <returns>
    /// A JObject
    /// </returns>
    private static JObject GetContent() {
        return JObject.Parse(File.ReadAllText(@"Resources\tetrominoes.json"));
    }
}
